from django.db.backends.base.introspection import BaseDatabaseIntrospection


class DatabaseIntrospection(BaseDatabaseIntrospection):
    def get_table_list(self, cursor):
        return self.connection.list_collection_names()

    def get_table_description(self, cursor, table_name):
        # Obtener la estructura de la colección (pseudo-esquema)
        return []



from django.db.backends.base.features import BaseDatabaseFeatures


class DatabaseFeatures(BaseDatabaseFeatures):
    """
    Features specific to MongoDB.
    """
    supports_transactions = False  # MongoDB no soporta transacciones ACID
    can_return_columns_from_insert = False
    supports_timezones = False



from django.db.backends.base.schema import BaseDatabaseSchemaEditor


class DatabaseSchemaEditor(BaseDatabaseSchemaEditor):
    sql_create_table = "/* MongoDB no requiere CREATE TABLE */"

    def create_model(self, model):
        """
        Create a collection for the given model.
        """
        collection = model._meta.db_table
        if collection not in self.connection.list_collection_names():
            self.connection.create_collection(collection)

        # Crear índices
        for field in model._meta.local_fields:
            if field.db_index:
                self.connection[collection].create_index(field.column)

    def delete_model(self, model):
        """
        Delete the collection for the given model.
        """
        collection_name = model._meta.db_table
        if collection_name in self.connection.connection.list_collection_names():
            self.connection.connection.drop_collection(collection_name)

    def add_field(self, model, field):
        if field.db_index:
            collection = model._meta.db_table
            self.connection[collection].create_index(field.column)



from bson import ObjectId
from django.core.exceptions import ValidationError


def validate_objectid(value):
    """
    Validate if a value is a valid ObjectId.
    """
    if not ObjectId.is_valid(value):
        raise ValidationError(f"'{value}' is not a valid ObjectId.")


def convert_to_objectid(value):
    """
    Convert a string to an ObjectId.
    """
    if isinstance(value, str) and ObjectId.is_valid(value):
        return ObjectId(value)
    return value


def get_collection_name(model):
    """
    Get the MongoDB collection name for a Django model.
    """
    return model._meta.db_table


def format_mongo_filter(django_filter):
    """
    Convert a Django filter to a MongoDB filter.
    """
    mongo_filter = {}
    for key, value in django_filter.items():
        if '__' in key:
            field_name, operator = key.split('__')
            mongo_operator = f'${operator}'
            mongo_filter[field_name] = {mongo_operator: value}
        else:
            mongo_filter[key] = value
    return mongo_filter



from django.db.models.sql.compiler import SQLCompiler


class MongoCompiler(SQLCompiler):
    def as_mongo(self):
        query = self.query
        collection = query.model._meta.db_table
        where = self._translate_where(query.where)
        return {
            'collection': collection,
            'filter': where,
            'projection': self._get_projection(),
            'limit': query.low_mark,
            'skip': query.high_mark,
        }

    def _translate_where(self, node):
        # Translate Django's WHERE clauses to MongoDB filters
        pass

    def _get_projection(self):
        return {field.column: 1 for field in self.query.select}
